#
# cbspline.tcl   Cubic Basis Spline  by zhuo
#
# Usage:  Do the following in your script.
#   0) Place this code with pkgIndex.tcl  in a Tcl library directory.
#   1) Import this package into your script by:
#       package require cbspline
#   2) As a preparation, create "bases" data by specifying the number 
#      of division; for instance, if you want to have 8 interpolated
#      points for one control point, pass 8 to this function:
#       set bases8 [cbspline::makeBases 8]
#   3) Now you can get an interpolated list out of the original, by:
#       set result [cbspline::interpolate $controlPoints $bases8]
#   4) For 2-dimensional polylines, do interpolation for x and y 
#      coordinates independently;

namespace eval cbspline {
}

######################################################################
# generate bases according to the number of division  ..
# return value (for the case nDiv = 10)
# { #body
#   b0 b1 b2 b3  # for t = 0.0
#   b0 b1 b2 b3  # for t = 0.1
#      :
#   b0 b1 b2 b3  # for t = 0.9
# }
# { b0 b1 b2 b3  } # for t = 1.0 (tail)

######################################################################
proc cbspline::makeBases {nDiv} {
    set div_1 [expr {1.0/$nDiv}]
    for { set idx 0 } { $idx <= $nDiv } { incr idx } {
	set t1 [expr {$idx * $div_1}]
	set t2 [expr {$t1*$t1}]
	set t3 [expr {$t2*$t1}]
	lappend bases \
	    [expr {(-$t3 + 3.0*$t2 - 3.0*$t1 + 1.0)/6.0}] \
	    [expr {(3.0*$t3 - 6.0*$t2 + 4.0)/6.0}] \
	    [expr {(-3.0*$t3 + 3.0*$t2 + 3.0 * $t1 + 1.0)/6.0}] \
	    [expr {$t3/6.0}]
    }
    return [list [lrange $bases 0 end-4] [lrange $bases end-3 end]]
}

#  *  *  x1   x2    x3    x4    x5  *  *
#  0  1   2                5 
proc cbspline::interpolate { list bases } {
    set basisBody [lindex $bases 0]
    set basisTail [lindex $bases 1]
    set nPnts [llength $list]
    set list [concat [lindex $list 0] [lindex $list 0] $list [lindex $list end] [lindex $list end]]
    
    for {set idx 0} {$idx <= $nPnts} {incr idx} {
	foreach {c0 c1 c2 c3} [lrange $list $idx [expr {$idx+3}]] {
	    foreach {b0 b1 b2 b3} $basisBody {
		lappend out [expr {$c0*$b0+$c1*$b1+$c2*$b2+$c3*$b3}]
	    }
	}
    }
    foreach {b0 b1 b2 b3} $basisTail {
	lappend out [expr {$c0*$b0+$c1*$b1+$c2*$b2+$c3*$b3}]
    }
    return $out
}

package provide cbspline 1.00


